@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,14 +149,11 @@ int SFtp::Do()
       const char *prog=Query("connect-program",hostname);
       if(!prog || !prog[0])
 	 prog="ssh -a -x";
-      char *a=alloca_strdup(prog);
-      ArgV *cmd=new ArgV;
-      for(a=strtok(a," "); a; a=strtok(0," "))
-	 cmd->Add(a);
+      ArgV args;
       if(!strchr(init,'/'))
       {
 	 if(init[0])
-	    cmd->Add("-s");   // run ssh2 subsystem
+	    args.Add("-s");   // run ssh2 subsystem
 	 // sftpd does not have a greeting
 	 received_greeting=true;
       }
@@ -164,20 +161,21 @@ int SFtp::Do()
 	 init=xstring::cat("echo SFTP: >&2;",init,NULL);
       if(user)
       {
-	 cmd->Add("-l");
-	 cmd->Add(user);
+	 args.Add("-l");
+	 args.Add(user);
       }
       if(portname)
       {
-	 cmd->Add("-p");
-	 cmd->Add(portname);
+	 args.Add("-p");
+	 args.Add(portname);
       }
-      cmd->Add(hostname);
+      args.Add(hostname);
       if(init[0])
-	 cmd->Add(init);
-      xstring_ca cmd_str(cmd->Combine(0));
+	 args.Add(init);
+      xstring_ca cmd_q(args.CombineShellQuoted(0));
+      xstring& cmd_str=xstring::cat(prog," ",cmd_q.get(),NULL);
       LogNote(9,"%s (%s)",_("Running connect program"),cmd_str.get());
-      ssh=new PtyShell(cmd);
+      ssh=new PtyShell(cmd_str);
       ssh->UsePipes();
       state=CONNECTING;
       timeout_timer.Reset();
@@ -448,7 +446,7 @@ SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
    if(res!=UNPACK_SUCCESS)
       return res;
 
-   Log::global->Format(9,"<--- got a packet, length=%d, type=%d(%s), id=%u\n",
+   LogRecvF(9,"got a packet, length=%d, type=%d(%s), id=%u\n",
       probe.GetLength(),probe.GetPacketType(),probe.GetPacketTypeText(),probe.GetID());
 
    switch(probe.GetPacketType())
@@ -526,7 +524,7 @@ void SFtp::SendRequest(Packet *request,Expect::expect_t tag,int i)
 {
    request->SetID(ssh_id++);
    request->ComputeLength();
-   Log::global->Format(9,"---> sending a packet, length=%d, type=%d(%s), id=%u\n",
+   LogSendF(9,"sending a packet, length=%d, type=%d(%s), id=%u\n",
       request->GetLength(),request->GetPacketType(),request->GetPacketTypeText(),request->GetID());
    request->Pack(send_buf.get_non_const());
    PushExpect(new Expect(request,tag,i));
@@ -600,9 +598,14 @@ void SFtp::SendRequest()
 	 SetError(NOT_SUPP);
 	 break;
       }
-      unsigned options=SSH_FXF_RENAME_NATIVE;
-      if(QueryBool("xfer:clobber",0))
-	 options|=SSH_FXF_RENAME_OVERWRITE;
+      unsigned options=0;
+      if(rename_f) {
+	 options=SSH_FXF_RENAME_OVERWRITE;
+	 if(protocol_version<5) {
+	    // overwrite is not supported, remove the target explicitly
+	    SendRequest(new Request_REMOVE(WirePath(file1)),Expect::IGNORE);
+	 }
+      }
       SendRequest(new Request_RENAME(WirePath(file),WirePath(file1),
 			options,protocol_version),Expect::DEFAULT);
       state=WAITING;
@@ -821,7 +824,7 @@ void SFtp::HandleExpect(Expect *e)
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
 	 }
-	 if(mode==CHANGE_DIR && GetExpectCount(Expect::CWD)==0)
+	 if(mode==CHANGE_DIR && !HasExpect(Expect::CWD))
 	 {
 	    cwd.Set(file);
 	    eof=true;
@@ -873,7 +876,7 @@ void SFtp::HandleExpect(Expect *e)
 	    d->GetData(&b,&s);
 	    LogNote(9,"data packet: pos=%lld, size=%d",(long long)r->pos,s);
 	    file_buf->Put(b,s);
-	    if(d->Eof())
+	    if(d->Eof() || eof)
 	       goto eof;
 	    if(r->len > unsigned(s))   // received less than requested?
 	    {
@@ -935,7 +938,7 @@ void SFtp::HandleExpect(Expect *e)
 	       file_set->Add(info);
 	    }
 	 }
-	 if(r->Eof())
+	 if(r->Eof() || eof)
 	    goto eof;
       }
       else
@@ -949,7 +952,7 @@ void SFtp::HandleExpect(Expect *e)
 		  LogNote(9,"eof");
 	       eof=true;
 	       state=DONE;
-	       if(file_buf && ooo_chain.count()==0)
+	       if(file_buf && ooo_chain.count()==0 && expect_queue.count()==0)
 		  file_buf->PutEOF();
 	       break;
 	    }
@@ -997,8 +1000,10 @@ void SFtp::HandleExpect(Expect *e)
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
       {
-	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK)
+	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK) {
+	    TrySuccess();
 	    break;
+	 }
       }
       SetError(NO_FILE,reply);
       break;
@@ -1021,6 +1026,7 @@ void SFtp::HandleExpect(Expect *e)
 
 void SFtp::RequestMoreData()
 {
+   Enter(this);
    if(mode==RETRIEVE) {
       int req_len=size_read;
       SendRequest(new Request_READ(handle,request_pos,req_len),Expect::DATA);
@@ -1028,6 +1034,7 @@ void SFtp::RequestMoreData()
    } else if(mode==LIST || mode==LONG_LIST) {
       SendRequest(new Request_READDIR(handle),Expect::DATA);
    }
+   Leave(this);
 }
 
 int SFtp::HandleReplies()
@@ -1055,7 +1062,7 @@ int SFtp::HandleReplies()
       }
    }
 
-   if(ooo_chain.count()==0 && eof && file_buf && !file_buf->Eof())
+   if(ooo_chain.count()==0 && eof && file_buf && !file_buf->Eof() && !HasExpect(Expect::DATA))
       file_buf->PutEOF();
 
    if(recv_buf->Size()<4)
@@ -1111,6 +1118,7 @@ SFtp::Expect *SFtp::FindExpectExclusive(Packet *p)
       e->reply=p;
    return e;
 }
+
 void SFtp::CloseExpectQueue()
 {
    for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
@@ -1137,12 +1145,12 @@ void SFtp::CloseExpectQueue()
    }
 }
 
-int SFtp::GetExpectCount(Expect::expect_t tag)
+bool SFtp::HasExpect(Expect::expect_t tag)
 {
-   int count=0;
    for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
-      count+=(e->tag==tag);
-   return count;
+      if(e->tag==tag)
+	 return true;
+   return false;
 }
 
 Glob *SFtp::MakeGlob(const char *pat)
@@ -1160,7 +1168,7 @@ int SFtp::Read(Buffer *buf,int size)
       return error_code;
    if(mode==CLOSED)
       return 0;
-   if(state==DONE)
+   if(state==DONE && !(file_buf && file_buf->Size()>0))
       return 0;	  // eof
    if(state==FILE_RECV)
    {
@@ -1172,7 +1180,10 @@ int SFtp::Read(Buffer *buf,int size)
 	 if(entity_size<0 || request_pos<entity_size || RespQueueSize()<2)
 	    RequestMoreData();
       }
+   }
 
+   if(file_buf && file_buf->Size()>0)
+   {
       const char *buf1;
       int size1;
       file_buf->Get(&buf1,&size1);
@@ -1204,7 +1215,9 @@ int SFtp::Write(const void *buf,int size)
       return(0);
 
    Resume();
+   Enter(this);
    Do();
+   Leave(this);
    if(Error())
       return(error_code);
 
@@ -1226,7 +1239,6 @@ int SFtp::Write(const void *buf,int size)
    if(size<=0)
       return 0;
    file_buf->Put(static_cast<const char*>(buf),size);
-   TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
@@ -1344,23 +1356,6 @@ bool SFtp::SameLocationAs(const FileAccess *fa) const
    return true;
 }
 
-void SFtp::Cleanup()
-{
-   if(hostname==0)
-      return;
-
-   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
-      fo->CleanupThis();
-
-   CleanupThis();
-}
-void SFtp::CleanupThis()
-{
-   if(mode!=CLOSED)
-      return;
-   Disconnect();
-}
-
 void SFtp::Reconfig(const char *name)
 {
    super::Reconfig(name);
@@ -1467,6 +1462,7 @@ const char *SFtp::Reply_STATUS::GetCodeText()
       "Connection lost",
       "Operation not supported",
       "Invalid handle",
+      "No such path",
       "File already exists",
       "Write protect",
       "No media",
@@ -1502,7 +1498,7 @@ void SFtp::SetError(int code,const Packet *reply)
    }
    Reply_STATUS *status=(Reply_STATUS*)reply;
    const char *message=status->GetMessage();
-   if(message)
+   if(message && *message)
    {
       SetError(code,utf8_to_lc(message));
       return;
@@ -2267,6 +2263,7 @@ int SFtpListInfo::Do()
 	 {
 	    if(file->filetype==file->SYMLINK && follow_symlinks)
 	    {
+	       file->filetype=file->UNKNOWN;
 	       file->defined &= ~(file->SIZE|file->DATE|file->SYMLINK_DEF|file->MODE|file->TYPE|file->USER|file->GROUP);
 	       file->Need(file->SIZE|file->DATE|file->MODE|file->TYPE|file->USER|file->GROUP);
 	    }
