@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2019 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,7 +52,8 @@ static const char *FtpProxyValidate(xstring_c *p)
    if(url.user && !url.pass)
    {
       url.pass.set(GetPass(_("ftp:proxy password: ")));
-      p->set_allocated(url.Combine());
+      p->truncate();
+      url.CombineTo(*p);
    }
    return 0;
 }
@@ -200,6 +201,7 @@ static ResType lftp_vars[] = {
    {"ftp:catch-size",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:charset",		 "",	  ResMgr::CharsetValidate,0},
    {"ftp:client",		 PACKAGE "/" VERSION,0,0},
+   {"ftp:compressed-re",	 "\\.(bz2|[glrsx7]z|lzma|lz[hox]|[ai]ce|apk|ar[cj]|cab|cfs|dar|[je]ar|lha|isz|pak|rar|sitx?|t(gz|bz2|lz)|tar\\.(gz|xz|bz2|lzma)|war|zipx?|zoo|[arz0][0-9][0-9])$",ResMgr::ERegExpValidate,ResMgr::NoClosure},
    {"ftp:device-prefix",	 "no",	  ResMgr::BoolValidate,0},
    {"ftp:fix-pasv-address",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:ignore-pasv-address",	 "no",	  ResMgr::BoolValidate,0},
@@ -212,6 +214,7 @@ static ResType lftp_vars[] = {
    {"ftp:lang",			 "",	  0,0},
    {"ftp:list-empty-ok",	 "no",	  0,0},
    {"ftp:list-options",		 "",	  0,0},
+   {"ftp:mode-z-level",		 "6",	  ResMgr::UNumberValidate,0},
    {"ftp:nop-interval",		 "120",   ResMgr::UNumberValidate,0},
    {"ftp:passive-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:port-range",		 "full",  ResMgr::RangeValidate,0},
@@ -222,14 +225,15 @@ static ResType lftp_vars[] = {
    {"ftp:rest-list",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:rest-stor",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:timezone",		 "GMT",   0,0},
+   {"ftp:too-many-re",		 "(Too many|No more) connections",ResMgr::ERegExpValidate,0},
    {"ftp:skey-allow",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:skey-force",		 "no",    ResMgr::BoolValidate,0},
    {"ftp:netkey-allow",		 "yes",   ResMgr::BoolValidate,0},
 #if USE_SSL
    {"ftp:ssl-allow",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:ssl-force",		 "no",	  ResMgr::BoolValidate,0},
-   {"ftp:ssl-protect-data",	 "no",	  ResMgr::BoolValidate,0},
-   {"ftp:ssl-protect-fxp",	 "no",    ResMgr::BoolValidate,0},
+   {"ftp:ssl-protect-data",	 "yes",	  ResMgr::BoolValidate,0},
+   {"ftp:ssl-protect-fxp",	 "yes",    ResMgr::BoolValidate,0},
    {"ftp:ssl-protect-list",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:ssl-auth",		 "TLS",   AuthArgValidate,0},
    {"ftp:ssl-allow-anonymous",	 "no",	  ResMgr::BoolValidate,0},
@@ -244,13 +248,14 @@ static ResType lftp_vars[] = {
    {"ftp:sync-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:trust-feat",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:use-abor",		 "yes",   ResMgr::BoolValidate,0},
-   {"ftp:use-allo",		 "yes",   ResMgr::BoolValidate,0},
+   {"ftp:use-allo",		 "no",    ResMgr::BoolValidate,0},
    {"ftp:use-feat",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-fxp",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-hftp",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-mdtm",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-mdtm-overloaded",	 "no",	  ResMgr::BoolValidate,0},
-   {"ftp:use-mlsd",		 "no",    ResMgr::BoolValidate,0},
+   {"ftp:use-mlsd",		 "yes",   ResMgr::BoolValidate,0},
+   {"ftp:use-mode-z",		 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:use-pret",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-chmod",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-idle",	 "no",    ResMgr::BoolValidate,0},
@@ -281,6 +286,7 @@ static ResType lftp_vars[] = {
    {"hftp:use-head",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-mkcol",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-propfind",	 "no",	  ResMgr::BoolValidate,0},
+   {"hftp:use-range",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-type",		 "yes",   ResMgr::BoolValidate,0},
    {"http:accept",		 "*/*",   0,0},
@@ -294,6 +300,7 @@ static ResType lftp_vars[] = {
    {"http:proxy",		 "",	  HttpProxyValidate,0},
    {"http:use-mkcol",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-propfind",	 "no",    ResMgr::BoolValidate,0},
+   {"http:use-range",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"http:user-agent",		 PACKAGE "/" VERSION,0,0},
    {"http:cookie",		 "",	  0,0},
@@ -308,14 +315,14 @@ static ResType lftp_vars[] = {
    {"net:idle",			 "3m",	  ResMgr::TimeIntervalValidate,0},
    {"net:limit-max",		 "0",	  ResMgr::UNumberValidate,0},
    {"net:limit-rate",		 "0:0",   ResMgr::UNumberPairValidate,0},
-   {"net:limit-total-max",	 "0",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
-   {"net:limit-total-rate",	 "0:0",   ResMgr::UNumberPairValidate,ResMgr::NoClosure},
+   {"net:limit-total-max",	 "0",	  ResMgr::UNumberValidate,0},
+   {"net:limit-total-rate",	 "0:0",   ResMgr::UNumberPairValidate,0},
    {"net:max-retries",		 "1000",  ResMgr::UNumberValidate,0},
    {"net:persist-retries",	 "0",	  ResMgr::UNumberValidate,0},
    {"net:no-proxy",		 "",	  0,ResMgr::NoClosure},
-   {"net:reconnect-interval-base","30",	  ResMgr::UNumberValidate,0},
+   {"net:reconnect-interval-base","15",	  ResMgr::UNumberValidate,0},
    {"net:reconnect-interval-multiplier","1.5",ResMgr::FloatValidate,0},
-   {"net:reconnect-interval-max","600",	  ResMgr::UNumberValidate,0},
+   {"net:reconnect-interval-max","300",	  ResMgr::UNumberValidate,0},
    {"net:socket-buffer",	 "0",	  ResMgr::UNumberValidate,0},
    {"net:socket-maxseg",	 "0",	  ResMgr::UNumberValidate,0},
    {"net:socket-bind-ipv4",	 "",	  ResMgr::IPv4AddrValidate,0},
@@ -324,15 +331,16 @@ static ResType lftp_vars[] = {
 #endif
    {"net:timeout",		 "5m",	  ResMgr::TimeIntervalValidate,0},
    {"net:connection-limit",	 "0",	  ResMgr::UNumberValidate,0},
+   {"net:connection-limit-timer","5m",	  ResMgr::TimeIntervalValidate,0},
    {"net:connection-takeover",	 "yes",   ResMgr::BoolValidate,0},
 
    {"mirror:sort-by",		 "name",  SortByValidate,ResMgr::NoClosure},
    {"mirror:order",		 "*.sfv *.sig *.md5* *.sum * */", 0,ResMgr::NoClosure},
    {"mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure},
-   {"mirror:parallel-transfer-count", "1",ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"mirror:parallel-transfer-count", "0",ResMgr::UNumberValidate,0},
    {"mirror:exclude-regex",	 "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure},
    {"mirror:include-regex",	 "",	  ResMgr::ERegExpValidate,ResMgr::NoClosure},
-   {"mirror:use-pget-n",	 "1",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
+   {"mirror:use-pget-n",	 "0",	  ResMgr::UNumberValidate,0},
    {"mirror:set-permissions",	 "yes",   ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:dereference",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:skip-noaccess",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
@@ -352,12 +360,14 @@ static ResType lftp_vars[] = {
 
    {"file:charset",		 "",	  ResMgr::CharsetValidate,ResMgr::NoClosure},
    {"file:use-lock",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"file:use-fallocate",	 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
 
    {"dns:cache-enable",		 "yes",	  ResMgr::BoolValidate,0},
    {"dns:cache-expire",		 "1h",	  ResMgr::TimeIntervalValidate,0},
    {"dns:cache-size",		 "256",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"dns:fatal-timeout",	 "7d",	  ResMgr::TimeIntervalValidate,0},
    {"dns:max-retries",		 "1000",  ResMgr::UNumberValidate,0},
+   {"dns:name",			 "",	  0,ResMgr::HasClosure},
 #if INET6
 # define DEFAULT_ORDER "inet6 inet"
 #else
@@ -380,12 +390,21 @@ static ResType lftp_vars[] = {
    {"xfer:destination-directory","",	  0,0},
    {"xfer:verify",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:verify-command",	 "",	  ResMgr::FileExecutable,0},
-   {"xfer:log",			 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:auto-rename",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
-   {"xfer:log-file",              "",     ResMgr::FileCreatable,ResMgr::NoClosure},
    {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"xfer:use-temp-file",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:temp-file-name",	 ".in.*", 0,ResMgr::NoClosure},
+   {"xfer:timeout",		 "1d",	  ResMgr::TimeIntervalValidate,ResMgr::NoClosure},
+   {"xfer:clobber",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:make-backup",		 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:keep-backup",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
+   {"xfer:backup-suffix",	 "~%Y%m%d%H%M%S~",0,ResMgr::NoClosure},
+   {"xfer:parallel",		 "1",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
+
+   // deprecated settings
+   {"xfer:log",		   "log:enabled/xfer",   0,ResMgr::AliasValidate},
+   {"xfer:log-file",	   "log:file/xfer",	0,ResMgr::AliasValidate},
+   {"xfer:max-log-size",   "log:max-size/xfer",	0,ResMgr::AliasValidate},
 
 #if USE_SSL
    {"ssl:ca-file",		 "",	  ResMgr::FileReadable,ResMgr::NoClosure},
@@ -455,21 +474,6 @@ void ResType::ClassInit()
    if(no_proxy)
       Set("net:no-proxy",0,no_proxy);
 
-#if INET6
-   // check if ipv6 is really supported
-   int s=socket(AF_INET6,SOCK_STREAM,IPPROTO_TCP);
-   if(s==-1 && (errno==EINVAL
-#ifdef EAFNOSUPPORT
-      || errno==EAFNOSUPPORT
-#endif
-   ))
-   {
-      Set("dns:order",0,"inet");
-   }
-   if(s!=-1)
-      close(s);
-#endif // INET6
-
    const char *module_path=getenv("LFTP_MODULE_PATH");
    if(module_path)
       Set("module:path",0,module_path);
@@ -488,5 +492,10 @@ void ResType::ClassInit()
    if(time_style && *time_style)
       Set("cmd:time-style",0,time_style);
 
-   Set("xfer:verify-command",0,PKGDATADIR"/verify-file");
+   SetDefault("xfer:verify-command",0,PKGDATADIR"/verify-file");
+
+   const char *ctx="xfer";
+   SetDefault("log:enabled",ctx,"yes");
+   SetDefault("log:show-time",ctx,"yes");
+   SetDefault("log:file",ctx,dir_file(get_lftp_data_dir(),"transfer_log"));
 }
